/*
**  CXSC is a C++ library for eXtended Scientific Computing 
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany
**
**  MPI communication package for STL vectors
**  of C-XSC data types and C-XSC Taylor arithmetic types
**
**  Copyright (C) 2004-2007 Markus Grimmer
**
**  markus.grimmer@math.uni-wuppertal.de
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * @file vector_mpicomm.hpp
 * 
 * MPI communication package for STL vectors
 * of C-XSC data types and C-XSC Taylor arithmetic types:
 * Header.
 *
 * Copyright (C) 2004-2007 Markus Grimmer
 *
 * Incorporated Routines:
 *
 * <ul>
 * <li> (see module cxsc_mpicomm_templ) </li>
 * </ul>
 *
 * Incorporated Types:
 *
 * <ul>
 * <li> vector </li>
 * </ul>
 */

// Remark: templates have to be explicitly instantiated!
// Remark: This strategy should work for all element data types that can be sent
//         as MPI_Send(Pointer,int,int,MPI_Comm)!
//         MPI_Send originally has the interface
//         MPI_Send (Pointer,int,MPI_Datatype,int,int,MPI_Comm).

#ifndef _VECTOR_MPICOMM_INCLUDE
#define _VECTOR_MPICOMM_INCLUDE

#include <mpi.h>
#include <vector>

// Taylor expansion modules (itaylor, dim2taylor)

#include "itaylor.hpp"
#include "dim2taylor.hpp"

using namespace std;

// MPI-Routinen fr STL-Vector

/**
 * @name Pack and Unpack for vector
 */
//@{
template<class T> 
int MPI_Pack (vector<T> &, void *, int, int *,  MPI_Comm);
template<class T> 
int MPI_Unpack (void*, int, int *, vector<T> &, MPI_Comm);
//@}

/**
 * @name Point-to-Point communication functions for vector
 */
//@{
template<class T>
int MPI_Send(vector<T> &, int, int, MPI_Comm);
template<class T>
int MPI_Bsend(vector<T> &, int, int, MPI_Comm);
template<class T>
int MPI_Ssend(vector<T> &, int, int, MPI_Comm);
template<class T>
int MPI_Rsend(vector<T> &, int, int, MPI_Comm);

template<class T>
int MPI_Isend(vector<T> &, int, int, MPI_Comm, MPI_Request*);
template<class T>
int MPI_Ibsend(vector<T> &, int, int, MPI_Comm, MPI_Request*);
template<class T>
int MPI_Issend(vector<T> &, int, int, MPI_Comm, MPI_Request*);
template<class T>
int MPI_Irsend(vector<T> &, int, int, MPI_Comm, MPI_Request*);

template<class T> 
int MPI_Recv (vector<T> &, int, int, MPI_Comm, MPI_Status*);
//@}

/**
 * @name Collective communication functions for vector
 */
//@{
template<class T>
int MPI_Bcast(vector<T>&, int, MPI_Comm);
template<class T>
int MPI_Bcast(vector<T>&, int, int, int, int, int, MPI_Comm);
//@}

#endif
